#!/usr/bin/env python
import logging
import nose
import re
import describe_it as di
import unittest
import itertools

log = logging.getLogger('nose.plugins.describe_it')


class DescribeItPlugin(nose.plugins.Plugin):
    name = 'describe-it'

    def wantModule(self, module):
        return re.search('spec$', module.__name__)

    def wantFile(self, file):
        return re.search('spec\.py$', file)

    def loadTestsFromModule(self, module):
        is_spec_module = re.search('spec$', module.__name__)
        if is_spec_module:
            test_cases = (create_testcases_for_context(c)
                          for c in di.get_contexts_for_module(module))
            return itertools.chain(*test_cases)


class ContextTestCase(unittest.TestCase):

    def __init__(self, context, it_fn):
        self.context = context
        self.it_fn = it_fn
        unittest.TestCase.__init__(self, methodName='run_test')

    def run_test(self):
        self.context.run_it(self.it_fn)

    def __str__(self):
        return '{0}: {1}'.format(str(self.context), self.it_fn.__name__)


def create_testcases_for_context(context):
    def create_it(it_fn):
        return ContextTestCase(context, it_fn)
    return map(create_it, context.it_fns)


if __name__ == "__main__":
    log.setLevel(logging.DEBUG)
    nose.main(addplugins=[DescribeItPlugin()])



import sys

registered_contexts = []
active_contexts = []


class Fixture(object):
    pass


def get_contexts_for_module(module):
    return (c for c in registered_contexts if c.module == module)


class Context(object):

    def __init__(self, describe_fn, parent):
        self.describe_fn = describe_fn
        self.parent = parent
        self.it_fns = []
        self.before_each_fns = []

    @property
    def module(self):
        return sys.modules[self.describe_fn.__module__]

    def add_it_fn(self, it_fn):
        self.it_fns.append(it_fn)

    def add_before_each(self, before_each_fn):
        self.before_each_fns.append(before_each_fn)

    def run_before_eaches(self):
        if self.parent:
            self.parent.run_before_eaches()

        for be in self.before_each_fns:
            be()

    def run_it(self, it_fn):
        self.run_before_eaches()
        it_fn()

    def __str__(self):
        if self.parent:
            return '{0} {1}'.format(
                str(self.parent), self.describe_fn.__name__)
        return self.describe_fn.__name__


def describe(describe_fn,
             registered_contexts=registered_contexts,
             active_contexts=active_contexts):

    parent = _current_active_context(active_contexts=active_contexts)

    context = Context(describe_fn=describe_fn, parent=parent)

    registered_contexts.append(context)

    active_contexts.append(context)
    describe_fn()
    active_contexts.pop()


def it(it_fn,
       registered_contexts=registered_contexts,
       active_contexts=active_contexts):

    context = _current_active_context(active_contexts=active_contexts)
    if not context:
        raise Exception("Adding an 'it' without telling me what you're "
                        "describing seems a bit silly.")

    context.add_it_fn(it_fn)


def before_each(before_each_fn,
                registered_contexts=registered_contexts,
                active_contexts=active_contexts):

    context = _current_active_context(active_contexts=active_contexts)
    if not context:
        raise Exception("Adding a 'before_each' without describing something "
                        "seems a bit silly.")

    context.add_before_each(before_each_fn)


def _current_active_context(active_contexts=active_contexts):
    return active_contexts[-1] if active_contexts else None



#!/usr/bin/env python
from setuptools import setup

setup(
    name='describe_it',
    version='0.1.6',
    description=('A nose plugin that supports writing describe/it style '
                 'unit tests with nested contexts'),
    long_description=open('README.rst').read(),
    license=open('LICENSE.txt').read(),
    url='https://github.com/joakimkarlsson/describe_it',
    author='Joakim Karlsson',
    author_email='joakim@jkarlsson.com',
    packages=['describe_it'],
    install_requires=['nose>=1.3.3'],

    tests_require=['nose>=1.3.3', 'mock>=1.0.1'],
    test_suite='nose.collector',

    entry_points={
        'nose.plugins.0.10': [
            'describe_it=describe_it.noseplugin:DescribeItPlugin'
        ]
    },

    keywords=['unit testing', 'tdd', 'bdd', 'nose']
)
